@@ -2,9 +2,14 @@
 """
 Regression tests for the units.format package
 """
+
+from __future__ import annotations
+
+import re
 import warnings
 from contextlib import nullcontext
 from fractions import Fraction
+from typing import TYPE_CHECKING, NamedTuple
 
 import numpy as np
 import pytest
@@ -12,14 +17,50 @@ from numpy.testing import assert_allclose
 
 from astropy import units as u
 from astropy.constants import si
-from astropy.units import PrefixUnit, Unit, UnitBase, UnitsWarning, dex
+from astropy.units import (
+    PrefixUnit,
+    Unit,
+    UnitBase,
+    UnitParserWarning,
+    UnitsWarning,
+    cds,
+    dex,
+)
 from astropy.units import format as u_format
 from astropy.units.utils import is_effectively_unity
+from astropy.utils.exceptions import AstropyDeprecationWarning
+
+if TYPE_CHECKING:
+    from collections.abc import Iterable
+
+
+class FormatStringPair(NamedTuple):
+    format: str
+    string: str
+
+
+class StringUnitPair(NamedTuple):
+    string: str
+    unit: UnitBase
+
+
+def list_format_string_pairs(*test_cases: tuple[str, str]) -> list[FormatStringPair]:
+    return [FormatStringPair(format, string) for format, string in test_cases]
+
+
+def list_string_unit_pairs(
+    *test_cases: tuple[Iterable[str], UnitBase],
+) -> list[StringUnitPair]:
+    return [
+        StringUnitPair(string, unit)
+        for strings, unit in test_cases
+        for string in strings
+    ]
 
 
 @pytest.mark.parametrize(
-    "strings, unit",
-    [
+    "test_pair",
+    list_string_unit_pairs(
         (["m s", "m*s", "m.s"], u.m * u.s),
         (["m/s", "m*s**-1", "m /s", "m / s", "m/ s"], u.m / u.s),
         (["m**2", "m2", "m**(2)", "m**+2", "m+2", "m^(+2)"], u.m**2),
@@ -35,13 +76,11 @@ from astropy.units.utils import is_effectively_unity
         (["mag(ct/s)"], u.MagUnit(u.ct / u.s)),
         (["dex"], u.dex),
         (["dex(cm s**-2)", "dex(cm/s2)"], u.DexUnit(u.cm / u.s**2)),
-    ],
+    ),
+    ids=lambda x: x.string,
 )
-def test_unit_grammar(strings, unit):
-    for s in strings:
-        print(s)
-        unit2 = u_format.Generic.parse(s)
-        assert unit2 == unit
+def test_unit_grammar(test_pair: StringUnitPair):
+    assert u_format.Generic.parse(test_pair.string) == test_pair.unit
 
 
 @pytest.mark.parametrize(
@@ -49,13 +88,12 @@ def test_unit_grammar(strings, unit):
 )
 def test_unit_grammar_fail(string):
     with pytest.raises(ValueError):
-        print(string)
         u_format.Generic.parse(string)
 
 
 @pytest.mark.parametrize(
-    "strings, unit",
-    [
+    "test_pair",
+    list_string_unit_pairs(
         (["0.1nm"], u.AA),
         (["mW/m2"], u.Unit(u.erg / u.cm**2 / u.s)),
         (["mW/(m2)"], u.Unit(u.erg / u.cm**2 / u.s)),
@@ -94,13 +132,13 @@ def test_unit_grammar_fail(string):
         (["[cm/s2]"], dex(u.cm / u.s**2)),
         (["[K]"], dex(u.K)),
         (["[-]"], dex(u.dimensionless_unscaled)),
-    ],
+        (["eps0/mu0"], cds.eps0 / cds.mu0),
+        (["a0.s"], cds.a0 * u.s),
+    ),
+    ids=lambda x: x.string,
 )
-def test_cds_grammar(strings, unit):
-    for s in strings:
-        print(s)
-        unit2 = u_format.CDS.parse(s)
-        assert unit2 == unit
+def test_cds_grammar(test_pair: StringUnitPair):
+    assert u_format.CDS.parse(test_pair.string) == test_pair.unit
 
 
 @pytest.mark.parametrize(
@@ -132,7 +170,6 @@ def test_cds_grammar(strings, unit):
 )
 def test_cds_grammar_fail(string):
     with pytest.raises(ValueError):
-        print(string)
         u_format.CDS.parse(string)
 
 
@@ -149,8 +186,8 @@ def test_cds_log10_dimensionless():
 # These examples are taken from the EXAMPLES section of
 # https://heasarc.gsfc.nasa.gov/docs/heasarc/ofwg/docs/general/ogip_93_001/
 @pytest.mark.parametrize(
-    "strings, unit",
-    [
+    "test_pair",
+    list_string_unit_pairs(
         (
             ["count /s", "count/s", "count s**(-1)", "count / s", "count /s "],
             u.count / u.s,
@@ -215,13 +252,11 @@ def test_cds_log10_dimensionless():
             ],
             (u.count / u.s) * (1.0 / (u.pixel * u.s)),
         ),
-    ],
+    ),
+    ids=lambda x: x.string,
 )
-def test_ogip_grammar(strings, unit):
-    for s in strings:
-        print(s)
-        unit2 = u_format.OGIP.parse(s)
-        assert unit2 == unit
+def test_ogip_grammar(test_pair: StringUnitPair):
+    assert u_format.OGIP.parse(test_pair.string) == test_pair.unit
 
 
 @pytest.mark.parametrize(
@@ -237,21 +272,83 @@ def test_ogip_grammar(strings, unit):
 )
 def test_ogip_grammar_fail(string):
     with pytest.raises(ValueError):
-        print(string)
         u_format.OGIP.parse(string)
 
 
-class RoundtripBase:
-    deprecated_units = set()
+@pytest.mark.xfail(reason="'acos' is not understood by astropy", raises=ValueError)
+def test_ogip_unusable_function():
+    u_format.OGIP.parse("acos(m)**2")
 
+
+@pytest.mark.parametrize("string", ["sqrt(m)**3", "sqrt(m**3)", "(sqrt(m))**3"])
+def test_ogip_sqrt(string):
+    # Regression test for #16743 - sqrt(m)**3 caused a ValueError
+    assert u_format.OGIP.parse(string) == u.m ** Fraction(3, 2)
+
+
+@pytest.mark.parametrize(
+    "string,message,unit",
+    [
+        pytest.param(
+            "m(s)**2",
+            (
+                r"^if 'm\(s\)\*\*2' was meant to be a multiplication, "
+                r"it should have been written as 'm \(s\)\*\*2'.$"
+            ),
+            u.m * u.s**2,
+            id="m(s)**2",
+        ),
+        pytest.param(
+            "m(s)",
+            (
+                r"^if 'm\(s\)' was meant to be a multiplication, "
+                r"it should have been written as 'm \(s\)'.$"
+            ),
+            u.m * u.s,
+            id="m(s)",
+        ),
+    ],
+)
+def test_ogip_invalid_multiplication(string, message, unit):
+    # Regression test for #16749
+    with pytest.warns(UnitParserWarning, match=message):
+        assert u_format.OGIP.parse(string) == unit
+
+
+@pytest.mark.parametrize(
+    "string,unit,power",
+    [
+        pytest.param("s**-1", u.s**-1, "-1", id="int_unit_power"),
+        pytest.param("m**-2.0", u.m**-2, "-2.0", id="float_unit_power"),
+        pytest.param("10**-3 kg", u.g, "-3", id="int_scale_power"),
+    ],
+)
+def test_ogip_negative_exponent_parenthesis(string, unit, power):
+    # Regression test for #16788 - negative powers require parenthesis
+    with pytest.warns(
+        UnitParserWarning,
+        match=(
+            r"^negative exponents must be enclosed in parenthesis\. "
+            rf"Expected '\*\*\({power}\)' instead of '\*\*{power}'\.$"
+        ),
+    ):
+        assert u_format.OGIP.parse(string) == unit
+
+
+def test_ogip_ohm():
+    # Regression test for #17200 - OGIP converted u.ohm to 'V / A'
+    assert u_format.OGIP.to_string(u.ohm) == "ohm"
+
+
+class RoundtripBase:
     def check_roundtrip(self, unit, output_format=None):
         if output_format is None:
-            output_format = self.format_
+            output_format = self.format_.name
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")  # Same warning shows up multiple times
             s = unit.to_string(output_format)
 
-        if s in self.deprecated_units:
+        if s in self.format_._deprecated_units:
             with pytest.warns(UnitsWarning, match="deprecated") as w:
                 a = Unit(s, format=self.format_)
             assert len(w) == 1
@@ -269,7 +366,7 @@ class RoundtripBase:
 
 
 class TestRoundtripGeneric(RoundtripBase):
-    format_ = "generic"
+    format_ = u_format.Generic
 
     @pytest.mark.parametrize(
         "unit",
@@ -278,6 +375,7 @@ class TestRoundtripGeneric(RoundtripBase):
             for unit in u.__dict__.values()
             if (isinstance(unit, UnitBase) and not isinstance(unit, PrefixUnit))
         ],
+        ids=str,
     )
     def test_roundtrip(self, unit):
         self.check_roundtrip(unit)
@@ -286,16 +384,12 @@ class TestRoundtripGeneric(RoundtripBase):
 
 
 class TestRoundtripVOUnit(RoundtripBase):
-    format_ = "vounit"
-    deprecated_units = u_format.VOUnit._deprecated_units
+    format_ = u_format.VOUnit
 
     @pytest.mark.parametrize(
         "unit",
-        [
-            unit
-            for unit in u_format.VOUnit._units.values()
-            if (isinstance(unit, UnitBase) and not isinstance(unit, PrefixUnit))
-        ],
+        [u for u in u_format.VOUnit._units.values() if not isinstance(u, PrefixUnit)],
+        ids=str,
     )
     def test_roundtrip(self, unit):
         self.check_roundtrip(unit)
@@ -304,31 +398,24 @@ class TestRoundtripVOUnit(RoundtripBase):
 
 
 class TestRoundtripFITS(RoundtripBase):
-    format_ = "fits"
-    deprecated_units = u_format.Fits._deprecated_units
+    format_ = u_format.FITS
 
     @pytest.mark.parametrize(
         "unit",
-        [
-            unit
-            for unit in u_format.Fits._units.values()
-            if (isinstance(unit, UnitBase) and not isinstance(unit, PrefixUnit))
-        ],
+        [u for u in u_format.FITS._units.values() if not isinstance(u, PrefixUnit)],
+        ids=str,
     )
     def test_roundtrip(self, unit):
         self.check_roundtrip(unit)
 
 
 class TestRoundtripCDS(RoundtripBase):
-    format_ = "cds"
+    format_ = u_format.CDS
 
     @pytest.mark.parametrize(
         "unit",
-        [
-            unit
-            for unit in u_format.CDS._units.values()
-            if (isinstance(unit, UnitBase) and not isinstance(unit, PrefixUnit))
-        ],
+        [u for u in u_format.CDS._units.values() if not isinstance(u, PrefixUnit)],
+        ids=str,
     )
     def test_roundtrip(self, unit):
         self.check_roundtrip(unit)
@@ -339,7 +426,7 @@ class TestRoundtripCDS(RoundtripBase):
         self.check_roundtrip_decompose(unit)
 
     @pytest.mark.parametrize(
-        "unit", [u.dex(unit) for unit in (u.cm / u.s**2, u.K, u.Lsun)]
+        "unit", [u.dex(unit) for unit in (u.cm / u.s**2, u.K, u.Lsun)], ids=str
     )
     def test_roundtrip_dex(self, unit):
         string = unit.to_string(format="cds")
@@ -348,8 +435,7 @@ class TestRoundtripCDS(RoundtripBase):
 
 
 class TestRoundtripOGIP(RoundtripBase):
-    format_ = "ogip"
-    deprecated_units = u_format.OGIP._deprecated_units | {"d"}
+    format_ = u_format.OGIP
 
     @pytest.mark.parametrize(
         "unit",
@@ -358,11 +444,12 @@ class TestRoundtripOGIP(RoundtripBase):
             for unit in u_format.OGIP._units.values()
             if (isinstance(unit, UnitBase) and not isinstance(unit, PrefixUnit))
         ],
+        ids=str,
     )
     def test_roundtrip(self, unit):
-        if str(unit) in ("d", "0.001 Crab"):
-            # Special-case day, which gets auto-converted to hours, and mCrab,
-            # which the default check does not recognize as a deprecated unit.
+        if str(unit) == "0.001 Crab":
+            # Special-case mCrab, which the default check does not recognize
+            # as a deprecated unit.
             with pytest.warns(UnitsWarning):
                 s = unit.to_string(self.format_)
                 a = Unit(s, format=self.format_)
@@ -386,23 +473,17 @@ class TestRoundtripOGIP(RoundtripBase):
             self.check_roundtrip_decompose(unit)
 
 
-def test_fits_units_available():
-    u_format.Fits._units
-
-
-def test_vo_units_available():
-    u_format.VOUnit._units
-
-
-def test_cds_units_available():
-    u_format.CDS._units
+@pytest.mark.parametrize(
+    "unit_formatter_class,n_units",
+    [(u_format.FITS, 765), (u_format.VOUnit, 1297), (u_format.CDS, 3124)],
+)
+def test_units_available(unit_formatter_class, n_units):
+    assert len(unit_formatter_class._units) == n_units
 
 
 def test_cds_non_ascii_unit():
     """Regression test for #5350.  This failed with a decoding error as
     μas could not be represented in ascii."""
-    from astropy.units import cds
-
     with cds.enable():
         u.radian.find_equivalent_units(include_prefix_units=True)
 
@@ -425,24 +506,107 @@ def test_latex_scale():
 
 def test_latex_inline_scale():
     fluxunit = u.Unit(1.0e-24 * u.erg / (u.cm**2 * u.s * u.Hz))
-    latex_inline = r"$\mathrm{1 \times 10^{-24}\,erg" r"\,Hz^{-1}\,s^{-1}\,cm^{-2}}$"
+    latex_inline = r"$\mathrm{1 \times 10^{-24}\,erg\,Hz^{-1}\,s^{-1}\,cm^{-2}}$"
     assert fluxunit.to_string("latex_inline") == latex_inline
 
 
 @pytest.mark.parametrize(
-    "format_spec, string",
+    "format_spec, string, decomposed",
     [
-        ("generic", "erg / (cm2 s)"),
-        ("s", "erg / (cm2 s)"),
-        ("console", "  erg  \n ------\n s cm^2"),
-        ("latex", "$\\mathrm{\\frac{erg}{s\\,cm^{2}}}$"),
-        ("latex_inline", "$\\mathrm{erg\\,s^{-1}\\,cm^{-2}}$"),
-        (">20s", "       erg / (cm2 s)"),
+        ("generic", "erg / (Angstrom s cm2)", "1e+07 kg / (m s3)"),
+        ("s", "erg / (Angstrom s cm2)", "1e+07 kg / (m s3)"),
+        ("console", "erg Angstrom^-1 s^-1 cm^-2", "10000000 kg m^-1 s^-3"),
+        (
+            "latex",
+            r"$\mathrm{\frac{erg}{\mathring{A}\,s\,cm^{2}}}$",
+            r"$\mathrm{10000000\,\frac{kg}{m\,s^{3}}}$",
+        ),
+        (
+            "latex_inline",
+            r"$\mathrm{erg\,\mathring{A}^{-1}\,s^{-1}\,cm^{-2}}$",
+            r"$\mathrm{10000000\,kg\,m^{-1}\,s^{-3}}$",
+        ),
+        ("unicode", "erg Å⁻¹ s⁻¹ cm⁻²", "10000000 kg m⁻¹ s⁻³"),
+        (">25s", "   erg / (Angstrom s cm2)", "        1e+07 kg / (m s3)"),
+        ("cds", "erg.Angstrom-1.s-1.cm-2", "10000000kg.m-1.s-3"),
+        ("ogip", "erg / (angstrom s cm**2)", "1e+07 kg / (m s**3)"),
+        ("fits", "erg Angstrom-1 s-1 cm-2", "10**7 kg m-1 s-3"),
+        ("vounit", "erg.Angstrom**-1.s**-1.cm**-2", "10000000kg.m**-1.s**-3"),
+        # TODO: make fits and vounit less awful!
     ],
 )
-def test_format_styles(format_spec, string):
+def test_format_styles(format_spec, string, decomposed):
+    fluxunit = u.erg / (u.cm**2 * u.s * u.Angstrom)
+    if format_spec == "vounit":
+        # erg and Angstrom are deprecated in vounit.
+        with pytest.warns(UnitsWarning, match="deprecated"):
+            formatted = format(fluxunit, format_spec)
+    else:
+        formatted = format(fluxunit, format_spec)
+    assert formatted == string
+    # Decomposed mostly to test that scale factors are dealt with properly
+    # in the various formats.
+    assert format(fluxunit.decompose(), format_spec) == decomposed
+
+
+@pytest.mark.parametrize(
+    "format_spec, fraction, string, decomposed",
+    [
+        ("generic", False, "erg s-1 cm-2", "0.001 kg s-3"),
+        (
+            "console",
+            "multiline",
+            " erg  \n------\ns cm^2",
+            "      kg \n0.001 ---\n      s^3",
+        ),
+        ("console", "inline", "erg / (s cm^2)", "0.001 kg / s^3"),
+        ("unicode", "multiline", " erg \n─────\ns cm²", "      kg\n0.001 ──\n      s³"),
+        ("unicode", "inline", "erg / (s cm²)", "0.001 kg / s³"),
+        (
+            "latex",
+            False,
+            r"$\mathrm{erg\,s^{-1}\,cm^{-2}}$",
+            r"$\mathrm{0.001\,kg\,s^{-3}}$",
+        ),
+        (
+            "latex",
+            "inline",
+            r"$\mathrm{erg / (s\,cm^{2})}$",
+            r"$\mathrm{0.001\,kg / s^{3}}$",
+        ),
+        # TODO: make generic with fraction=False less awful!
+    ],
+)
+def test_format_styles_non_default_fraction(format_spec, fraction, string, decomposed):
     fluxunit = u.erg / (u.cm**2 * u.s)
-    assert format(fluxunit, format_spec) == string
+    assert fluxunit.to_string(format_spec, fraction=fraction) == string
+    assert fluxunit.decompose().to_string(format_spec, fraction=fraction) == decomposed
+
+
+@pytest.mark.parametrize("format_spec", ["generic", "cds", "fits", "ogip", "vounit"])
+def test_no_multiline_fraction(format_spec):
+    fluxunit = u.W / u.m**2
+    with pytest.raises(
+        ValueError,
+        match=(
+            f"^'{format_spec}' format only supports 'inline' fractions, "
+            r"not fraction='multiline'\.$"
+        ),
+    ):
+        fluxunit.to_string(format_spec, fraction="multiline")
+
+
+@pytest.mark.parametrize("format_spec", ["latex", "console", "unicode"])
+def test_unknown_fraction_style(format_spec):
+    fluxunit = u.W / u.m**2
+    with pytest.raises(
+        ValueError,
+        match=(
+            f"^'{format_spec}' format only supports 'inline' or 'multiline' fractions, "
+            r"not fraction='parrot'\.$"
+        ),
+    ):
+        fluxunit.to_string(format_spec, fraction="parrot")
 
 
 def test_flatten_to_known():
@@ -465,6 +629,21 @@ def test_console_out():
     u.Jy.decompose().to_string("console")
 
 
+@pytest.mark.parametrize(
+    "test_pair",
+    list_format_string_pairs(
+        ("generic", "10"),
+        ("console", "10"),
+        ("unicode", "10"),
+        ("cds", "10"),
+        ("latex", r"$\mathrm{10}$"),
+    ),
+    ids=lambda x: x.format,
+)
+def test_scale_only(test_pair: FormatStringPair):
+    assert u.Unit(10).to_string(test_pair.format) == test_pair.string
+
+
 def test_flexible_float():
     assert u.min._represents.to_string("latex") == r"$\mathrm{60\,s}$"
 
@@ -476,7 +655,7 @@ def test_fits_to_string_function_error():
     """
 
     with pytest.raises(TypeError, match="unit argument must be"):
-        u_format.Fits.to_string(None)
+        u_format.FITS.to_string(None)
 
 
 def test_fraction_repr():
@@ -541,14 +720,13 @@ def test_deprecated_did_you_mean_units():
     assert "Crab (deprecated)" in str(exc_info.value)
     assert "mCrab (deprecated)" in str(exc_info.value)
 
-    with pytest.warns(
-        UnitsWarning,
-        match=r".* Did you mean 0\.1nm, Angstrom "
-        r"\(deprecated\) or angstrom \(deprecated\)\?",
-    ) as w:
+    with pytest.raises(
+        ValueError,
+        match=(
+            r"Did you mean 0\.1nm, Angstrom \(deprecated\) or angstrom \(deprecated\)\?"
+        ),
+    ):
         u.Unit("ANGSTROM", format="vounit")
-    assert len(w) == 1
-    assert str(w[0].message).count("0.1nm") == 1
 
     with pytest.warns(UnitsWarning, match=r".* 0\.1nm\.") as w:
         u.Unit("angstrom", format="vounit")
@@ -559,8 +737,7 @@ def test_deprecated_did_you_mean_units():
 def test_fits_function(string):
     # Function units cannot be written, so ensure they're not parsed either.
     with pytest.raises(ValueError):
-        print(string)
-        u_format.Fits().parse(string)
+        u_format.FITS().parse(string)
 
 
 @pytest.mark.parametrize("string", ["mag(ct/s)", "dB(mW)", "dex(cm s**-2)"])
@@ -573,12 +750,11 @@ def test_vounit_function(string):
 
 
 def test_vounit_binary_prefix():
-    u.Unit("KiB", format="vounit") == u.Unit("1024 B")
-    u.Unit("Kibyte", format="vounit") == u.Unit("1024 B")
-    u.Unit("Kibit", format="vounit") == u.Unit("1024 B")
-    with pytest.warns(UnitsWarning) as w:
+    assert u.Unit("KiB", format="vounit") == u.Unit("1024 B")
+    assert u.Unit("Kibyte", format="vounit") == u.Unit("1024 B")
+    assert u.Unit("Kibit", format="vounit") == u.Unit("128 B")
+    with pytest.raises(ValueError, match="not supported by the VOUnit standard"):
         u.Unit("kibibyte", format="vounit")
-    assert len(w) == 1
 
 
 def test_vounit_unknown():
@@ -588,9 +764,18 @@ def test_vounit_unknown():
 
 
 def test_vounit_details():
-    with pytest.warns(UnitsWarning, match="deprecated") as w:
-        assert u.Unit("Pa", format="vounit") is u.Pascal
-    assert len(w) == 1
+    assert u.Unit("Pa", format="vounit") is u.Pascal
+    assert u.Unit("ka", format="vounit") == u.Unit("1000 yr")
+    assert u.Unit("pix", format="vounit") == u.Unit("pixel", format="vounit")
+
+    # Regression test for astropy/astroquery#2480
+    assert u.Unit("Sun", format="vounit") is u.Sun
+
+    # Test that adding a prefix to a simple units raises a warning
+    with pytest.warns(
+        UnitsWarning, match="Unit 'kdB' not supported by the VOUnit standard.*"
+    ):
+        u.Unit("kdB", format="vounit", parse_strict="warn")
 
     # The da- prefix is not allowed, and the d- prefix is discouraged
     assert u.dam.to_string("vounit") == "10m"
@@ -600,7 +785,7 @@ def test_vounit_details():
     with pytest.warns(UnitsWarning, match="deprecated"):
         flam = u.erg / u.cm / u.cm / u.s / u.AA
         x = u.format.VOUnit.to_string(flam)
-        assert x == "Angstrom**-1.cm**-2.erg.s**-1"
+        assert x == "erg.Angstrom**-1.s**-1.cm**-2"
         new_flam = u.format.VOUnit.parse(x)
         assert new_flam == flam
 
@@ -623,6 +808,37 @@ def test_vounit_scale_factor(unit, vounit, number, scale, voscale):
     assert x.to_string(format="vounit") == voscale + vounit
 
 
+@pytest.mark.parametrize(
+    "unit, vounit",
+    [
+        ("m s^-1", "m/s"),
+        ("s^-1", "1/s"),
+        ("100 s^-2", "100/s**2"),
+        ("kg m-1 s-2", "kg/(m.s**2)"),
+    ],
+)
+@pytest.mark.parametrize("fraction", [True, "inline"])
+def test_vounit_fraction(unit, vounit, fraction):
+    x = u.Unit(unit)
+    assert x.to_string(format="vounit", fraction=fraction) == vounit
+
+
+@pytest.mark.parametrize(
+    "unit, vounit",
+    [
+        ("m^2", "m**2"),
+        ("s^-1", "s**-1"),
+        ("s(0.333)", "s**(0.333)"),
+        ("s(-0.333)", "s**(-0.333)"),
+        ("s(1/3)", "s**(1/3)"),
+        ("s(-1/3)", "s**(-1/3)"),
+    ],
+)
+def test_vounit_power(unit, vounit):
+    x = u.Unit(unit)
+    assert x.to_string(format="vounit") == vounit
+
+
 def test_vounit_custom():
     x = u.Unit("'foo' m", format="vounit")
     x_vounit = x.to_string("vounit")
@@ -641,7 +857,7 @@ def test_vounit_custom():
 def test_vounit_implicit_custom():
     # Yikes, this becomes "femto-urlong"...  But at least there's a warning.
     with pytest.warns(UnitsWarning) as w:
-        x = u.Unit("furlong/week", format="vounit")
+        x = u.Unit("furlong/week", format="vounit", parse_strict="warn")
     assert x.bases[0]._represents.scale == 1e-15
     assert x.bases[0]._represents.bases[0].name == "urlong"
     assert len(w) == 2
@@ -673,11 +889,11 @@ def test_vounit_implicit_custom():
 def test_fits_scale_factor(scale, number, string):
     x = u.Unit(scale + " erg/(s cm**2 Angstrom)", format="fits")
     assert x == number * (u.erg / u.s / u.cm**2 / u.Angstrom)
-    assert x.to_string(format="fits") == string + " Angstrom-1 cm-2 erg s-1"
+    assert x.to_string(format="fits") == string + " erg Angstrom-1 s-1 cm-2"
 
     x = u.Unit(scale + "*erg/(s cm**2 Angstrom)", format="fits")
     assert x == number * (u.erg / u.s / u.cm**2 / u.Angstrom)
-    assert x.to_string(format="fits") == string + " Angstrom-1 cm-2 erg s-1"
+    assert x.to_string(format="fits") == string + " erg Angstrom-1 s-1 cm-2"
 
 
 def test_fits_scale_factor_errors():
@@ -695,18 +911,35 @@ def test_fits_scale_factor_errors():
     assert x.to_string(format="fits") == "10**2 erg"
 
 
-def test_double_superscript():
-    """Regression test for #5870, #8699, #9218; avoid double superscripts."""
-    assert (u.deg).to_string("latex") == r"$\mathrm{{}^{\circ}}$"
-    assert (u.deg**2).to_string("latex") == r"$\mathrm{deg^{2}}$"
-    assert (u.arcmin).to_string("latex") == r"$\mathrm{{}^{\prime}}$"
-    assert (u.arcmin**2).to_string("latex") == r"$\mathrm{arcmin^{2}}$"
-    assert (u.arcsec).to_string("latex") == r"$\mathrm{{}^{\prime\prime}}$"
-    assert (u.arcsec**2).to_string("latex") == r"$\mathrm{arcsec^{2}}$"
-    assert (u.hourangle).to_string("latex") == r"$\mathrm{{}^{h}}$"
-    assert (u.hourangle**2).to_string("latex") == r"$\mathrm{hourangle^{2}}$"
-    assert (u.electron).to_string("latex") == r"$\mathrm{e^{-}}$"
-    assert (u.electron**2).to_string("latex") == r"$\mathrm{electron^{2}}$"
+@pytest.mark.parametrize(
+    "unit, latex, unicode",
+    [
+        (u.deg, r"$\mathrm{{}^{\circ}}$", "°"),
+        (u.deg**2, r"$\mathrm{deg^{2}}$", "deg²"),
+        (u.arcmin, r"$\mathrm{{}^{\prime}}$", "′"),
+        (u.arcmin**2, r"$\mathrm{arcmin^{2}}$", "arcmin²"),
+        (u.arcsec, r"$\mathrm{{}^{\prime\prime}}$", "″"),
+        (u.arcsec**2, r"$\mathrm{arcsec^{2}}$", "arcsec²"),
+        (u.hourangle, r"$\mathrm{{}^{h}}$", "ʰ"),
+        (u.hourangle**2, r"$\mathrm{hourangle^{2}}$", "hourangle²"),
+        (u.electron, r"$\mathrm{e^{-}}$", "e⁻"),
+        (u.electron**2, r"$\mathrm{electron^{2}}$", "electron²"),
+    ],
+)
+def test_double_superscript(unit, latex, unicode):
+    """Regression test for #5870, #8699, #9218, #14403; avoid double superscripts."""
+    assert unit.to_string("latex") == latex
+    assert unit.to_string("unicode") == unicode
+
+
+def test_no_prefix_superscript():
+    """Regression test for gh-911 and #14419."""
+    assert u.mdeg.to_string("latex") == r"$\mathrm{mdeg}$"
+    assert u.narcmin.to_string("latex") == r"$\mathrm{narcmin}$"
+    assert u.parcsec.to_string("latex") == r"$\mathrm{parcsec}$"
+    assert u.mdeg.to_string("unicode") == "mdeg"
+    assert u.narcmin.to_string("unicode") == "narcmin"
+    assert u.parcsec.to_string("unicode") == "parcsec"
 
 
 @pytest.mark.parametrize(
@@ -738,7 +971,7 @@ def test_powers(power, expected):
         ("\N{MICRO SIGN}g", u.microgram),
         ("\N{GREEK SMALL LETTER MU}g", u.microgram),
         ("g\N{MINUS SIGN}1", u.g ** (-1)),
-        ("m\N{SUPERSCRIPT MINUS}\N{SUPERSCRIPT ONE}", 1 / u.m),
+        ("m\N{SUPERSCRIPT MINUS}\N{SUPERSCRIPT ONE}", u.m**-1),
         ("m s\N{SUPERSCRIPT MINUS}\N{SUPERSCRIPT ONE}", u.m / u.s),
         ("m\N{SUPERSCRIPT TWO}", u.m**2),
         ("m\N{SUPERSCRIPT PLUS SIGN}\N{SUPERSCRIPT TWO}", u.m**2),
@@ -767,6 +1000,18 @@ def test_powers(power, expected):
 def test_unicode(string, unit):
     assert u_format.Generic.parse(string) == unit
     assert u.Unit(string) == unit
+    # Should work in composites too.
+    assert u.Unit(f"{string}/s") == unit / u.s
+    assert u.Unit(f"m {string}") == u.m * unit
+    assert u.Unit(f"{string} {string}") == unit**2
+    # Not obvious that "°2" should be "deg**2", but not easy to reject,
+    # and "R♃²" should work.  But don't run on examples with a space or that
+    # already end in a number.
+    if re.match(r"^\S*[^\d⁰¹²³⁴⁵⁶⁷⁸⁹]$", string):
+        assert u.Unit(f"{string}2") == unit**2
+    assert u.Unit(f"{string}/{string}") == u.dimensionless_unscaled
+    # Finally, check round-trip
+    assert u.Unit(unit.to_string("unicode")) == unit
 
 
 @pytest.mark.parametrize(
@@ -794,3 +1039,79 @@ def test_parse_error_message_for_output_only_format(format_):
 def test_unknown_parser():
     with pytest.raises(ValueError, match=r"Unknown.*unicode'\] for output only"):
         u.Unit("m", format="foo")
+
+
+def test_celsius_fits():
+    assert u.Unit("Celsius", format="fits") == u.deg_C
+    assert u.Unit("deg C", format="fits") == u.deg_C
+
+    # check that compounds do what we expect: what do we expect?
+    assert u.Unit("deg C kg-1", format="fits") == u.C * u.deg / u.kg
+    assert u.Unit("Celsius kg-1", format="fits") == u.deg_C / u.kg
+    assert u.deg_C.to_string("fits") == "Celsius"
+
+
+@pytest.mark.parametrize(
+    "test_pair",
+    list_format_string_pairs(
+        ("generic", "dB(1 / m)"),
+        ("latex", r"$\mathrm{dB}$$\mathrm{\left( \mathrm{\frac{1}{m}} \right)}$"),
+        ("latex_inline", r"$\mathrm{dB}$$\mathrm{\left( \mathrm{m^{-1}} \right)}$"),
+        ("console", "dB(m^-1)"),
+        ("unicode", "dB(m⁻¹)"),
+    ),
+    ids=lambda x: x.format,
+)
+def test_function_format_styles(test_pair: FormatStringPair):
+    dbunit = u.decibel(u.m**-1)
+    assert dbunit.to_string(test_pair.format) == test_pair.string
+    assert f"{dbunit:{test_pair.format}}" == test_pair.string
+
+
+@pytest.mark.parametrize(
+    "format_spec, fraction, string",
+    [
+        ("console", "multiline", "   1\ndB(-)\n   m"),
+        ("console", "inline", "dB(1 / m)"),
+        ("unicode", "multiline", "   1\ndB(─)\n   m"),
+        ("unicode", "inline", "dB(1 / m)"),
+        ("latex", False, r"$\mathrm{dB}$$\mathrm{\left( \mathrm{m^{-1}} \right)}$"),
+        ("latex", "inline", r"$\mathrm{dB}$$\mathrm{\left( \mathrm{1 / m} \right)}$"),
+    ],
+)
+def test_function_format_styles_non_default_fraction(format_spec, fraction, string):
+    dbunit = u.decibel(u.m**-1)
+    assert dbunit.to_string(format_spec, fraction=fraction) == string
+
+
+@pytest.mark.parametrize(
+    "test_pair",
+    list_format_string_pairs(
+        ("", "1"),
+        (".1g", "1"),
+        (".3g", "1"),
+        (".1e", "1.0"),
+        (".1f", "1.0"),
+        (".3e", "1.000"),
+    ),
+    ids=lambda x: repr(x.format),
+)
+def test_format_latex_one(test_pair: FormatStringPair):
+    # see https://github.com/astropy/astropy/issues/12571
+    assert (
+        u_format.Latex.format_exponential_notation(1, test_pair.format)
+        == test_pair.string
+    )
+
+
+def test_Fits_name_deprecation():
+    with pytest.warns(
+        AstropyDeprecationWarning,
+        match=(
+            r'^The class "Fits" has been renamed to "FITS" in version 7\.0\. '
+            r"The old name is deprecated and may be removed in a future version\.\n"
+            r"        Use FITS instead\.$"
+        ),
+    ):
+        from astropy.units.format import Fits
+    assert Fits is u.format.FITS
