@@ -13,6 +13,7 @@ import warnings
 
 import numpy as np
 from matplotlib import rcParams
+from matplotlib.ticker import Formatter
 
 from astropy import units as u
 from astropy.coordinates import Angle
@@ -57,6 +58,13 @@ CUSTOM_UNITS = {
 }
 
 
+def _fix_minus(labels: list[str], /) -> list[str]:
+    # correctly support axes.unicode_minus, but do it in a
+    # way that preserves arbitrary separators: only fix the leading character
+    # see https://github.com/astropy/astropy/issues/15898
+    return [Formatter.fix_minus(s[0]) + s[1:] for s in labels]
+
+
 class BaseFormatterLocator:
     """
     A joint formatter/locator.
@@ -99,7 +107,7 @@ class BaseFormatterLocator:
         if not values.unit.is_equivalent(self._unit):
             raise UnitsError(
                 "value should be in units compatible with "
-                "coordinate units ({}) but found {}".format(self._unit, values.unit)
+                f"coordinate units ({self._unit}) but found {values.unit}"
             )
         self._number = None
         self._spacing = None
@@ -189,6 +197,7 @@ class AngleFormatterLocator(BaseFormatterLocator):
         self._decimal = decimal
         self._sep = None
         self.show_decimal_unit = show_decimal_unit
+        self._alwayssign = False
 
         super().__init__(
             values=values,
@@ -254,6 +263,10 @@ class AngleFormatterLocator(BaseFormatterLocator):
         if value is None:
             return
 
+        self._alwayssign = value.startswith("+")
+        if self._alwayssign:
+            value = value[1:]
+
         if DMS_RE.match(value) is not None:
             self._decimal = False
             self._format_unit = u.degree
@@ -490,9 +503,10 @@ class AngleFormatterLocator(BaseFormatterLocator):
                 fields=fields,
                 sep=sep,
                 format=fmt,
+                alwayssign=self._alwayssign,
             ).tolist()
 
-            return string
+            return _fix_minus(string)
         else:
             return []
 
@@ -511,15 +525,12 @@ class ScalarFormatterLocator(BaseFormatterLocator):
         unit=None,
         format_unit=None,
     ):
-        if unit is not None:
-            unit = unit
-            format_unit = format_unit or unit
-        elif spacing is not None:
-            unit = spacing.unit
-            format_unit = format_unit or spacing.unit
-        elif values is not None:
-            unit = values.unit
-            format_unit = format_unit or values.unit
+        if unit is None:
+            if spacing is not None:
+                unit = spacing.unit
+            elif values is not None:
+                unit = values.unit
+        format_unit = format_unit or unit
 
         super().__init__(
             values=values,
@@ -637,10 +648,14 @@ class ScalarFormatterLocator(BaseFormatterLocator):
             else:
                 precision = self._precision
 
-            return [
-                ("{0:." + str(precision) + "f}").format(x.to_value(self._format_unit))
-                for x in values
-            ]
+            return _fix_minus(
+                [
+                    ("{0:." + str(precision) + "f}").format(
+                        x.to_value(self._format_unit)
+                    )
+                    for x in values
+                ]
+            )
 
         else:
             return []
